@@ -30,8 +30,10 @@
 #include <string.h>
 #include "system_defs.h"
 #include "net.h"
-#include "ivf_format.h"
+#include "network.h"
 #include "externs.h"
+#include "psfigs.h"
+#include "asg_module.h"
 
 /*---------------------------------------------------------------
  * Forward References
@@ -41,8 +43,33 @@
 void add_in_out();
 void add_special();
 
+#ifdef TCL_WRAPPER
+
+/*---------------------------------------------------------------
+ * my_calloc: replacement for C utility "calloc", using the Tcl
+ * routine Tcl_Alloc followed by bzero to initialize memory.
+ *---------------------------------------------------------------
+ */
+
+void *
+my_calloc(size_t nmemb, size_t size)
+{
+   void *memloc;
+   size_t memsiz = nmemb * size;
+
+   memloc = (void *)Tcl_Alloc(memsiz);
+   if (memloc) bzero(memloc, memsiz);
+   return memloc;
+}
+
+#endif  /* TCL_WRAPPER */
+
 /*---------------------------------------------------------------
  * error: generic error reporting, should have more on line numbers etc.
+ *
+ * Warning:  This routine name conflicts with a C utility, so
+ * it is necessary to order the libraries correctly when linking
+ * to avoid an incorrect link.
  *---------------------------------------------------------------
  */
 
@@ -170,13 +197,10 @@ net *get_net(name)
     nlist *n;
 
     for (n = nets; n != NULL ; n = n->next)
-    {
 	if (!strcmp(n->this->name, name))
-	{
 	    return(n->this);
-	}
-    }
-    error ("get_net, net used before it is declared ", name);
+
+    /* error ("get_net, net used before it is declared ", name); */
     return(NULL);
 }
 
@@ -315,20 +339,19 @@ module *newobject(gate_name, gate_type)
     {
 	error("newobject: too many modules, change MAX_MOD and recompile",
 	      gate_name);
-	exit(1);
+	/* exit(1); */
+	return NULL;
     }
     
     object = (module *)calloc(1, sizeof(module));
     object->index = module_count++;
     object->name = strdup(gate_name);
     object->type = strdup(gate_type);
-    object->dly_type = INERTIAL_DELAY;
-    object->delay = 0;
     object->x_pos = 0;
     object->y_pos = 0;    
     object->flag = UNSEEN;
     object->placed = UNPLACED;
-    object->rot = LEFT;		/* asuume no rotation */
+    object->rot = LEFT;		/* assme no rotation */
     object->primary_in = NULL;
     object->primary_out = NULL;
     object->terms = NULL;
@@ -400,6 +423,24 @@ void addinout(name, index, noOfTerms)
     
 	else add_in_out(name, index, noOfTerms, INOUT);
 }
+
+void addoutin(name, index, noOfTerms)
+    char *name;
+    int index;
+    int noOfTerms;
+{
+/* MODIFICATION 6-11-90 Skip "_DUMMY_SIGNAL_", "$1" and $0" strings... */
+	if ((!strcmp(name, STR_DUMMY_SIGNAL)) ||
+	    (!strcmp(name, SIG_ZERO)) ||
+	    (!strcmp(name, SIG_ONE)) ||
+	    (!strcmp(name, STR_GUARD_SIGNAL)))	    /* See "system_defs.h" */
+	{
+	    add_special(name, index, noOfTerms,OUTIN);
+	}
+    
+	else add_in_out(name, index, noOfTerms,OUTIN);
+}
+
     
 /* ------------------------------------------------------------------------
  * relative_term_location: determine where a signal should be located on the module:
@@ -449,6 +490,7 @@ int relative_term_location(termNo, termsOnEdge, edgeLength, spacesPerPin)
  if space == 1, the progression is 1, 3, 5, 7 ...
  if space == 2, the progression is 1, 4, 7, 10 ...
 */
+
 /* ------------------------------------------------------------------------
  * add_in_out: add a signal to the list of terminals for a module
  * ------------------------------------------------------------------------
@@ -466,7 +508,6 @@ void add_in_out(name, index, noOfTerms, dir)
     /* find this net in the global database */
     tn = get_net(name);
     tt->nt = tn;    
-
     tt->mod = curobj_module;
     tt->name = (char *)calloc(1, TERM_NAME_SIZE);
 
@@ -490,6 +531,7 @@ case OUT:
 	    tt->mod->primary_out = tt;	       /* hack to see that all mods have a PO */
 	    break;
 case INOUT:
+
 	    sprintf(tt->name, "inout_%d", index);
 	    tt->side = UP;     /* MODIFICATION 20-10-89 stf (labels are up) */
 	                       /* MOD 6-11-90 (SHOULD BE 'UP'- SCED LABELS AREN'T USED */
@@ -499,6 +541,17 @@ case INOUT:
 	    tt->x_pos = relative_term_location(index, noOfTerms, tt->mod->x_size, SPACES_PER_PIN);
 	    tt->y_pos = tt->mod->y_size + 1;   /* Put at top of icon... */
 	    break;
+
+case OUTIN:
+	    sprintf(tt->name, "inout_%d", index);
+	    tt->side = BOTTOM;     /* MODIFICATION 20-10-89 stf (labels are up) */
+	                       /* MOD 6-11-90 (SHOULD BE 'UP'- SCED LABELS AREN'T USED */
+	    tt->type = OUTIN;
+    
+	    /* MOD 5-2-91 INOUT's are distributed accross the top of the icon */
+	    tt->x_pos = relative_term_location(index, noOfTerms, tt->mod->x_size, SPACES_PER_PIN);
+	    tt->y_pos = -1;   /* Put at top of icon... */
+	    break;
 default: error("add_in_out: terminal with bad direction", name);
  }
     
@@ -507,11 +560,74 @@ default: error("add_in_out: terminal with bad direction", name);
 	/* tie the term back into the list of terms on this net */
 
          tn->terms = (tlist *)concat_list(tt, tn->terms);
-
 }
 
-/* OLD:tt->y_pos = ((index % 2) == 1) ? tt->mod->y_size/2 - index/2	         */
-/*	                              : tt->mod->y_size/2 +  index/2 + 1;        */
+/* ------------------------------------------------------------------------
+ * add_xc_term: add a signal to the list of terminals for a module
+ * based on the XCircuit object for gate "name".
+ * ------------------------------------------------------------------------
+ */
+
+void add_xc_term(type, pinName, netName, index)
+    char *type;
+    char *pinName;
+    char *netName;
+    int index;
+{
+    tlist *tl;
+    term *tt;
+    net *tn;
+    objinstptr xc_inst;
+    int result, x, y, dist, mindist;
+    
+    /* Get the xcircuit object name corresponding to the module */
+    if (NameToObject(type, &xc_inst, FALSE) == NULL) return;
+
+    /* Find the location of the pin named "pinName" relative to the object position */
+    result = NameToPinLocation(xc_inst, pinName, &x, &y);
+    if (result < 0) {
+       add_in_out(netName, index, 1, 0);	/* backup behavior */
+       return;					/* Error---pin name not found */
+    }
+
+    tt = (term *)calloc(1, sizeof(term));
+
+    /* find this net in the global database */
+    tn = get_net(netName);
+    tt->nt = tn;    
+
+    tt->mod = curobj_module;
+    tt->name = (char *)calloc(1, TERM_NAME_SIZE);
+
+    /* Treat all (analog) signals as "inout", but determine which side	*/
+    /* is closest to the pin position.					*/
+    sprintf(tt->name, "inout_%d", index);
+    tt->type = INOUT;
+    tt->x_pos = x;
+    tt->y_pos = y;
+
+    tt->side = LEFT;
+    mindist = x - xc_inst->bbox.lowerleft.x;
+    dist = y - xc_inst->bbox.lowerleft.y;
+    if (dist < mindist) {
+       mindist = dist;
+       tt->side = BOTTOM;
+    }
+    dist = xc_inst->bbox.lowerleft.y + xc_inst->bbox.height - y;
+    if (dist < mindist) {
+       mindist = dist;
+       tt->side = UP;
+    }
+    dist = xc_inst->bbox.lowerleft.x + xc_inst->bbox.width - x;
+    if (dist < mindist)
+       tt->side = RIGHT;
+    
+    curobj_module->terms = (tlist *)concat_list(tt, curobj_module->terms);
+
+    /* tie the term back into the list of terms on this net */
+
+    tn->terms = (tlist *)concat_list(tt, tn->terms);
+}
 
 /* ------------------------------------------------------------------------
  * add_special: add a special terminal to a module
@@ -557,6 +673,17 @@ case INOUT:
 	    tt->x_pos = relative_term_location(index, noOfTerms, tt->mod->x_size, SPACES_PER_PIN);
 	    tt->y_pos = tt->mod->y_size + 1;   /* Put at top of icon... */
 	    break;
+
+case OUTIN:
+	    sprintf(tt->name, "inout_%d", index);
+	    tt->side = BOTTOM;     /* MODIFICATION 20-10-89 stf (labels are up) */
+	                       /* MOD 6-11-90 (SHOULD BE 'UP'- SCED LABELS AREN'T USED */
+	    tt->type = OUTIN;
+    
+	    /* MOD 5-2-91 INOUT's are distributed accross the top of the icon */
+	    tt->x_pos = relative_term_location(index, noOfTerms, tt->mod->x_size, SPACES_PER_PIN);
+	    tt->y_pos = -1;   /* Put at top of icon... */
+	    break;
 default: error("add_special: terminal with bad direction", name);
 }
     /* The TYPE is based on the signal name: */
@@ -697,7 +824,7 @@ int fix_pin_position(m, nn, x, y)
 }
 
 /* ------------------------------------------------------------------------
- * reset_default_icon_size - use the x,y dimensions from .ivf   added 3-92 (stf)
+ * reset_default_icon_size - use the x,y dimensions  added 3-92 (stf)
  *-------------------------------------------------------------------------
  */
 int reset_default_icon_size(m, x, y)
@@ -861,16 +988,7 @@ int reposition_all_terminals(m, incount, inoutcount, outcount, spacesPerPin)
     }
 }
 
-/* ------------------------------------------------------------------------
- * adddelay: add the simulation delay information to a module
- * ------------------------------------------------------------------------
- */
-void adddelay(time)
-    int time;
-{
-    curobj_module->delay = time;
-}
-    
+   
 /* ------------------------------------------------------------------------
  * add_port: add a terminal to the list of system terminals
  * this list is different from the other terminal lists:
@@ -1264,6 +1382,30 @@ void clip_null_gates()
     free_list(temp);
 }
 
+/*---------------------------------------------------------------
+ * str_end_copy
+ *---------------------------------------------------------------
+ */
+
+char *str_end_copy(new, old, n)
+    char *new, *old;
+    int n;
+{
+    /* Copy the last <n> characters from <old> to <new>. */
+    int i, len = strlen(old);
+    if (len > n)
+    {
+	strncpy(new, old, n);
+	for (i=0; i<n; i++)
+	{
+	    new[i] = old[len-n+i];
+	}
+	new[n] = '\0';
+    }
+    else strcpy(new, old);
+
+    return(new);
+}
 
 /* ------------------------------------------------------------------------
  * END OF FILE
