@@ -249,7 +249,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    int depth, lbus, netid, subnetid;
    buslist *sbus;
    pushlistptr seltop, nextptr;
-   objectptr nettop, thisobj;
+   objectptr nettop, thisobj, pschem;
    char *snew;
    stringpart *ppin;
 
@@ -307,10 +307,13 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
          while (depth-- > 0) 
 	    UPopCTM();
 
+	 /* get the primary schematic */
+	 pschem = (nettop->schemtype == SECONDARY) ? nettop->symschem : nettop;
+
 	 /* print the net name to the message window */
 
 	 if (netlist->subnets == 0) {
-	    ppin = nettopin(netlist->net.id, nettop, NULL);
+	    ppin = nettopin(netlist->net.id, pschem, NULL);
             snew = textprint(ppin, areastruct.topinstance);
 	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
             free(snew);
@@ -323,7 +326,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	       sbus = netlist->net.list + lbus;
 	       netid = sbus->netid;
 	       subnetid = sbus->subnetid;
-	       ppin = nettopin(netid, nettop, NULL);
+	       ppin = nettopin(netid, pschem, NULL);
                snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
 	       sprintf(sptr, "%s ", snew);
 	       sptr += strlen(snew) + 1;
